/*****************************************************************************
   BM1390GLV.ino

   Copyright (c) 2021 ROHM Co.,Ltd.

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
 ******************************************************************************/
#include <Arduino.h>
#include <HardwareSerial.h>
#include <Wire.h>
#include <BM1390GLV.h>

#define SYSTEM_BAUDRATE (115200)
#define INO_VERSION     ("1.0")
#define SYSTEM_WAIT     (200)       // ms
#define ERROR_WAIT      (1000)      // ms

BM1390GLV bm1390glv;

void error_func(int32_t result);

void setup()
{
    int32_t result;
    String  INOVersion, DriverVersion;

    Serial.begin(SYSTEM_BAUDRATE);
    while (!Serial) {
    }

    INOVersion = INO_VERSION;
    (void)Serial.print("BM1390GLV Sample Code Version : ");
    (void)Serial.println(INOVersion);

    DriverVersion = bm1390glv.get_driver_version();
    (void)Serial.print("BM1390GLV Driver Version : ");
    (void)Serial.println(DriverVersion);

    Wire.begin();

    result = bm1390glv.init();
    if (result == BM1390GLV_COMM_OK) {
        (void)bm1390glv.start();
    } else {
        error_func(result);
    }

    return;
}

void loop()
{
    int32_t result;
    float32 press, temp;

    result = bm1390glv.get_val(&press, &temp);
    if (result == BM1390GLV_COMM_OK) {
        (void)Serial.write("Pressure(hPa):Temperature(degree Celsius) = ");
        (void)Serial.print(press);
        (void)Serial.write(", ");
        (void)Serial.print(temp);
        (void)Serial.println();
    } else {
        error_func(result);
    }

    delay(SYSTEM_WAIT);

    return;
}

void error_func(int32_t result)
{
    uint8_t cnt;

    switch (result) {
        case BM1390GLV_COMM_ERROR:
            (void)Serial.println("Communication Error.");
            break;

        case BM1390GLV_WAI_ERROR:
            (void)Serial.println("ID Error.");
            break;

        default:
            (void)Serial.println("Unknown Error.");
            break;
    }

    (void)Serial.println("BM1390GLV Check System and Driver Parameter.");
    cnt = 0;
    while (1) {
        (void)Serial.print(".");
        if (cnt < 30) {
            cnt++;
        } else {
            cnt = 0;
            (void)Serial.println();
        }
        delay(ERROR_WAIT);
    }

    return;
}